package practice2;

import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

/**
 * Servlet obsluhujici formular. Rada veci neni dodelana (zejmena validace vstupu a
 * reseni pripadnych chyb).
 */
public class Practice2Servlet extends HttpServlet {
   
    /** 
     * Handles the HTTP <code>GET</code> method.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        try {
            // objekt dispecera - predavame rizeni na jsp stranku s formularem
            RequestDispatcher rd = request.getRequestDispatcher("/Practice2jsp.jsp");
            
            String newToken = "" + Math.random();
            String newTokenName = this.getClass().getName() + Math.random();
            
            String tokenName = request.getParameter("tokenName");
            String token = request.getParameter("token");
            
            // HTTP session - bude potreba pro ulozeni historie
            HttpSession session = request.getSession();
            String sessionToken = null;
            
            synchronized(session) {
            	sessionToken = (String) session.getAttribute(tokenName);
            	session.removeAttribute(tokenName);
            }
            
            
            // neobdrzeli jsme hodnoty z formulare - nedojde k zadnemu zpracovani
            if (request.getParameter("amount") == null || request.getParameter("amount").isEmpty() ||
            		sessionToken == null || token == null || !sessionToken.equals(token)) {
                session.setAttribute(newTokenName, newToken);
                request.setAttribute("token", newToken);
                request.setAttribute("tokenName", newTokenName);
                //response.sendRedirect("/");
                rd.forward(request, response);
                return;
            }
            
            // historii udrzujeme pres vice vlastnich requestu - a to v HTTP session!
            List<Transaction> history = (List<Transaction>) session.getAttribute("history");
            if (history == null) {
                history = new LinkedList<Transaction>();
                session.setAttribute("history", history);
            }
            
            // informace o prevodu zapouzdruje objekt Transaction
            Transaction t = new Transaction();
            t.setSrcAmount(Double.parseDouble(request.getParameter("amount")));
            t.setSrcCurrency(Currency.get(request.getParameter("srcCurrency")));
            t.setDstCurrency(Currency.get(request.getParameter("dstCurrency")));
            t.setDstAmount(t.getSrcCurrency().convertTo(t.getSrcAmount(), t.getDstCurrency()));

            // ulozeni konverze do historie
            history.add(t);
            
            // ulozeni potrebnych dat do request scope
            
            session.setAttribute(newTokenName, newToken);
            request.setAttribute("token", newToken);
            request.setAttribute("tokenName", newTokenName);
            request.setAttribute("transaction", t);
            request.setAttribute("history", history);
            
            
            // nakonec se rizeni preda na JSP formular
            //response.sendRedirect("/");
            rd.forward(request, response);
        }
        catch (Exception e) {
            System.err.println("" + e.getMessage());
        }
    } 

    /** 
     * Handles the HTTP <code>POST</code> method.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        doGet(request, response);
    }

    /** 
     * Returns a short description of the servlet.
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Currency converter with history";
    }

}